﻿using UnityEngine.TestTools;
using NUnit.Framework;
using System.Collections;
using Windows.Kinect;
using System.Runtime.InteropServices;
using KinectModule;

public class SourceBufferTests
{
    KinectAccessManager _KinectAccessManager = new KinectAccessManager();
    SourceBuffer _SourceBuffer = new SourceBuffer();

    [Test]
	public void SourceBufferTestsSimplePasses()
    {
        // Use the Assert class to test conditions.

        _KinectAccessManager.Access();
        Assert.IsTrue(_KinectAccessManager.IsAccess, "Kinect Access Error.");

        var sourceFrame = _KinectAccessManager.SourceFrame;
        _SourceBuffer.UpdateBuffers(sourceFrame);

        ColorBufferTest(sourceFrame);
        BodyIndexBufferTest(sourceFrame);
        DepthBufferTest(sourceFrame);
        BodyBufferTest(sourceFrame);

        _KinectAccessManager.Dispose();
    }

    private void ColorBufferTest(MultiSourceFrame sourceFrame)
    {
        var colorBuffer = new byte[SourceBuffer.ColorWidth * SourceBuffer.ColorHeight * 4];
        using (var colorFrame = sourceFrame.ColorFrameReference.AcquireFrame())
            if (colorFrame != null)
            {
                var ColorData = GCHandle.Alloc(colorBuffer, GCHandleType.Pinned);
                colorFrame.CopyConvertedFrameDataToIntPtr(ColorData.AddrOfPinnedObject(), (uint)colorBuffer.Length, ColorImageFormat.Rgba);
                ColorData.Free();
            }

        Assert.AreEqual(colorBuffer, _SourceBuffer.ColorBuffer, "ColorBufferTest failed.");
    }

    private void BodyIndexBufferTest(MultiSourceFrame sourceFrame)
    {
        var bodyIndexBuffer = new byte[SourceBuffer.DepthWidth * SourceBuffer.DepthHeight];
        using (var bodyIndexFrame = sourceFrame.BodyIndexFrameReference.AcquireFrame())
            if (bodyIndexFrame != null)
            {
                var BodyIndexData = GCHandle.Alloc(bodyIndexBuffer, GCHandleType.Pinned);
                bodyIndexFrame.CopyFrameDataToIntPtr(BodyIndexData.AddrOfPinnedObject(), (uint)bodyIndexBuffer.Length);
                BodyIndexData.Free();
            }

        Assert.AreEqual(bodyIndexBuffer, _SourceBuffer.BodyIndexBuffer, "BodyIndexBufferTest failed.");
    }

    private void DepthBufferTest(MultiSourceFrame sourceFrame)
    {
        var depthBuffer = new ushort[SourceBuffer.DepthWidth * SourceBuffer.DepthHeight];
        using (var depthFrame = sourceFrame.DepthFrameReference.AcquireFrame())
            if (depthFrame != null)
            {
                var DepthData = GCHandle.Alloc(depthBuffer, GCHandleType.Pinned);
                depthFrame.CopyFrameDataToIntPtr(DepthData.AddrOfPinnedObject(), (uint)depthBuffer.Length * sizeof(ushort));
                DepthData.Free();
            }

        Assert.AreEqual(depthBuffer, _SourceBuffer.DepthBuffer, "DepthBufferTest failed.");
    }

    private void BodyBufferTest(MultiSourceFrame sourceFrame)
    {
        var bodyBuffer = new Body[SourceBuffer.BodyCount];
        using (var bodyFrame = sourceFrame.BodyFrameReference.AcquireFrame())
            if (bodyFrame != null)
                bodyFrame.GetAndRefreshBodyData(bodyBuffer);

        Assert.AreEqual(bodyBuffer, _SourceBuffer.BodyBuffer, "BodyBufferTest failed.");
    }

    // A UnityTest behaves like a coroutine in PlayMode
    // and allows you to yield null to skip a frame in EditMode
    [UnityTest]
	public IEnumerator SourceBufferTestsWithEnumeratorPasses()
    {
		// Use the Assert class to test conditions.
		// yield to skip a frame
		yield return null;
	}
}
